@@ -46,6 +46,7 @@
  */
 
 #include <config.h>
+#include <gm_alloc.h>
 #include <stdio.h>
 #include <sys/time.h>
 #include <arpa/inet.h>
@@ -54,11 +55,11 @@
 #define MOD_GM_COMMON_H
 
 /* constants */
-#define GM_VERSION                 "1.4.14"
+#define GM_VERSION                 "1.5.5"
 #define GM_ENABLED                      1
 #define GM_DISABLED                     0
-#define GM_BUFFERSIZE               98304
-#define GM_MAX_OUTPUT               65536   /* must be ~30% below GM_BUFFERSIZE for base64/encryption */
+#define GM_BUFFERSIZE               65536
+#define GM_MAX_OUTPUT             1048576   /* limit plugin output size to 1mb */
 #define GM_LISTSIZE                   512
 #define GM_NEBTYPESSIZE                33   /* maximum number of neb types */
 
@@ -69,6 +70,10 @@
 #define GM_ERROR                        1
 #define GM_NO_EPN                      -1
 
+#define GM_PERFDATA_ALL                 2
+
+#define GM_EXIT_UNKNOWN               768   /* results in exit code 3 after processed by WEXITSTATUS() */
+
 /* log modes */
 #define GM_LOG_ERROR                   -1
 #define GM_LOG_INFO                     0
@@ -236,6 +241,9 @@ typedef struct mod_gm_opt_struct {
     int            use_perl_cache;                          /**< cache embedded perl scripts */
     char         * p1_file;                                 /**< path to p1 file, needed for embedded perl */
 #endif
+    char         * restrict_path[GM_LISTSIZE];              /**< list of path restrictions */
+    int            restrict_path_num;                       /**< number of path restrictions */
+    char         * restrict_command_characters;             /**< forbidden characters in command lines */
     int            workaround_rc_25;                        /**< optional workaround for plugins returning exit code 25 */
 /* send_gearman */
     int            timeout;                                 /**< timeout for waiting reading on stdin */
