@@ -85,7 +85,7 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
     neb_set_module_info( gearman_module_handle, NEBMODULE_MODINFO_LICENSE, "GPL v3" );
     neb_set_module_info( gearman_module_handle, NEBMODULE_MODINFO_DESC,    "distribute host/service checks and eventhandler via gearman" );
 
-    mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
+    mod_gm_opt = gm_malloc(sizeof(mod_gm_opt_t));
     set_default_options(mod_gm_opt);
 
     /* parse arguments */
@@ -106,7 +106,7 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
         gm_log( GM_LOG_ERROR, "mod_gearman needs BROKER_TIMED_EVENTS (%i) event_broker_options enabled to work\n", BROKER_TIMED_EVENTS );
         broker_option_errors++;
     }
-    if(    (    mod_gm_opt->perfdata == GM_ENABLED
+    if(    (    mod_gm_opt->perfdata != GM_DISABLED
              || mod_gm_opt->hostgroups_num > 0
              || mod_gm_opt->hosts == GM_ENABLED
            )
@@ -114,7 +114,7 @@ int nebmodule_init( int flags, char *args, nebmodule *handle ) {
         gm_log( GM_LOG_ERROR, "mod_gearman needs BROKER_HOST_CHECKS (%i) event_broker_options enabled to work\n", BROKER_HOST_CHECKS );
         broker_option_errors++;
     }
-    if(    (    mod_gm_opt->perfdata == GM_ENABLED
+    if(    (    mod_gm_opt->perfdata != GM_DISABLED
              || mod_gm_opt->servicegroups_num > 0
              || mod_gm_opt->services == GM_ENABLED
            )
@@ -190,7 +190,7 @@ static void register_neb_callbacks(void) {
     if ( mod_gm_opt->events == GM_ENABLED )
         neb_register_callback( NEBCALLBACK_EVENT_HANDLER_DATA, gearman_module_handle, 0, handle_eventhandler );
 
-    if ( mod_gm_opt->perfdata == GM_ENABLED ) {
+    if ( mod_gm_opt->perfdata != GM_DISABLED ) {
         if(process_performance_data == 0)
             gm_log( GM_LOG_INFO, "Warning: process_performance_data is disabled globally, cannot process performance data\n" );
         neb_register_callback( NEBCALLBACK_HOST_CHECK_DATA, gearman_module_handle, 0, handle_perfdata );
@@ -222,7 +222,7 @@ int nebmodule_deinit( int flags, int reason ) {
     if ( mod_gm_opt->events == GM_ENABLED )
         neb_deregister_callback( NEBCALLBACK_EVENT_HANDLER_DATA, gearman_module_handle );
 
-    if ( mod_gm_opt->perfdata == GM_ENABLED ) {
+    if ( mod_gm_opt->perfdata != GM_DISABLED ) {
         neb_deregister_callback( NEBCALLBACK_HOST_CHECK_DATA, gearman_module_handle );
         neb_deregister_callback( NEBCALLBACK_SERVICE_CHECK_DATA, gearman_module_handle );
     }
@@ -511,12 +511,6 @@ static int handle_host_check( int event_type, void *data ) {
         && hostdata->type != NEBTYPE_HOSTCHECK_SYNC_PRECHECK)
         return NEB_OK;
 
-    /* shouldn't happen - internal Nagios error */
-    if ( hostdata == 0 ) {
-        gm_log( GM_LOG_ERROR, "Host handler received NULL host data structure.\n" );
-        return NEB_OK;
-    }
-
     /* get objects and set target function */
     if((hst=hostdata->object_ptr)==NULL) {
         gm_log( GM_LOG_ERROR, "Host handler received NULL host object pointer.\n" );
@@ -625,16 +619,16 @@ static int handle_host_check( int event_type, void *data ) {
     /* orphaned check - submit fake result to mark host as orphaned */
     if(mod_gm_opt->orphan_host_checks == GM_ENABLED && check_options & CHECK_OPTION_ORPHAN_CHECK) {
         gm_log( GM_LOG_DEBUG, "host check for %s orphaned\n", hst->name );
-        if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 )
+        if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 )
             return NEBERROR_CALLBACKCANCEL;
         snprintf( temp_buffer,GM_BUFFERSIZE-1,"(host check orphaned, is the mod-gearman worker on queue '%s' running?)\n", target_queue);
         init_check_result(chk_result);
-        chk_result->host_name           = strdup( hst->name );
+        chk_result->host_name           = gm_strdup( hst->name );
         chk_result->scheduled_check     = TRUE;
         chk_result->reschedule_check    = TRUE;
         chk_result->output_file         = 0;
-        chk_result->output_file_fd      = -1;
-        chk_result->output              = strdup(temp_buffer);
+        chk_result->output_file_fp      = NULL;
+        chk_result->output              = gm_strdup(temp_buffer);
         chk_result->return_code         = mod_gm_opt->orphan_return;
         chk_result->check_options       = CHECK_OPTION_NONE;
         chk_result->object_check_type   = HOST_CHECK;
@@ -677,12 +671,6 @@ static int handle_svc_check( int event_type, void *data ) {
     if ( svcdata->type != NEBTYPE_SERVICECHECK_ASYNC_PRECHECK )
         return NEB_OK;
 
-    /* shouldn't happen - internal Nagios error */
-    if ( svcdata == 0 ) {
-        gm_log( GM_LOG_ERROR, "Service handler received NULL service data structure.\n" );
-        return NEBERROR_CALLBACKCANCEL;
-    }
-
     /* get objects and set target function */
     if((svc=svcdata->object_ptr)==NULL) {
         gm_log( GM_LOG_ERROR, "Service handler received NULL service object pointer.\n" );
@@ -804,18 +792,18 @@ static int handle_svc_check( int event_type, void *data ) {
     /* orphaned check - submit fake result to mark service as orphaned */
     if(mod_gm_opt->orphan_service_checks == GM_ENABLED && svc->check_options & CHECK_OPTION_ORPHAN_CHECK) {
         gm_log( GM_LOG_DEBUG, "service check for %s - %s orphaned\n", svc->host_name, svc->description );
-        if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 )
+        if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 )
             return NEBERROR_CALLBACKCANCEL;
         snprintf( temp_buffer,GM_BUFFERSIZE-1,"(service check orphaned, is the mod-gearman worker on queue '%s' running?)\n", target_queue);
         init_check_result(chk_result);
-        chk_result->host_name           = strdup( svc->host_name );
-        chk_result->service_description = strdup( svc->description );
+        chk_result->host_name           = gm_strdup( svc->host_name );
+        chk_result->service_description = gm_strdup( svc->description );
         chk_result->scheduled_check     = TRUE;
         chk_result->reschedule_check    = TRUE;
         chk_result->output_file         = 0;
-        chk_result->output_file_fd      = -1;
-        chk_result->output              = strdup(temp_buffer);
-        chk_result->return_code         = 2;
+        chk_result->output_file_fp      = NULL;
+        chk_result->output              = gm_strdup(temp_buffer);
+        chk_result->return_code         = mod_gm_opt->orphan_return;
         chk_result->check_options       = CHECK_OPTION_NONE;
         chk_result->object_check_type   = SERVICE_CHECK;
         chk_result->check_type          = SERVICE_CHECK_ACTIVE;
@@ -846,7 +834,7 @@ static int read_arguments( const char *args_orig ) {
         return GM_ERROR;
     }
 
-    args = strdup(args_orig);
+    args = gm_strdup(args_orig);
     args_c = args;
 
     while ( (ptr = strsep( &args, " " )) != NULL ) {
@@ -1061,8 +1049,8 @@ int handle_perfdata(int event_type, void *data) {
                     break;
                 }
 
-                hst = find_host(hostchkdata->host_name);
-                if(hst->process_performance_data == 0) {
+                hst = (host *) hostchkdata->object_ptr;
+                if(hst->process_performance_data == 0 && mod_gm_opt->perfdata != GM_PERFDATA_ALL) {
                     gm_log( GM_LOG_TRACE, "handle_perfdata() process_performance_data disabled for: %s\n", hst->name );
                     break;
                 }
@@ -1078,11 +1066,13 @@ int handle_perfdata(int event_type, void *data) {
                             "HOSTPERFDATA::%s\t"
                             "HOSTCHECKCOMMAND::%s!%s\t"
                             "HOSTSTATE::%d\t"
-                            "HOSTSTATETYPE::%d\n",
+                            "HOSTSTATETYPE::%d\n"
+                            "HOSTINTERVAL::%f\n\n",
                             (int)hostchkdata->timestamp.tv_sec,
                             hostchkdata->host_name, hostchkdata->perf_data,
                             hostchkdata->command_name, hostchkdata->command_args,
-                            hostchkdata->state, hostchkdata->state_type);
+                            hostchkdata->state, hostchkdata->state_type,
+                            hst->check_interval);
                 has_perfdata = TRUE;
             }
             break;
@@ -1096,8 +1086,8 @@ int handle_perfdata(int event_type, void *data) {
                 }
 
                 /* find the nagios service object for this service */
-                svc = find_service(srvchkdata->host_name, srvchkdata->service_description);
-                if(svc->process_performance_data == 0) {
+                svc = (service *) srvchkdata->object_ptr;
+                if(svc->process_performance_data == 0 && mod_gm_opt->perfdata != GM_PERFDATA_ALL) {
                     gm_log( GM_LOG_TRACE, "handle_perfdata() process_performance_data disabled for: %s - %s\n", svc->host_name, svc->description );
                     break;
                 }
@@ -1114,11 +1104,13 @@ int handle_perfdata(int event_type, void *data) {
                             "SERVICEPERFDATA::%s\t"
                             "SERVICECHECKCOMMAND::%s\t"
                             "SERVICESTATE::%d\t"
-                            "SERVICESTATETYPE::%d\n\n\n",
+                            "SERVICESTATETYPE::%d\n"
+                            "SERVICEINTERVAL::%f\n\n",
                             (int)srvchkdata->timestamp.tv_sec,
                             srvchkdata->host_name, srvchkdata->service_description,
                             srvchkdata->perf_data, svc->service_check_command,
-                            srvchkdata->state, srvchkdata->state_type);
+                            srvchkdata->state, srvchkdata->state_type,
+                            svc->check_interval);
                 temp_buffer[GM_BUFFERSIZE-1]='\x0';
                 has_perfdata = TRUE;
             }
