@@ -82,7 +82,7 @@ void *result_worker( void * data ) {
 /* put back the result into the core */
 void *get_results( gearman_job_st *job, void *context, size_t *result_size, gearman_return_t *ret_ptr ) {
     int wsize, transportmode;
-    char workload[GM_BUFFERSIZE];
+    char *workload;
     char *decrypted_data;
     char *decrypted_data_c;
 #ifdef GM_DEBUG
@@ -108,13 +108,14 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
 
     /* get the data */
     wsize = gearman_job_workload_size(job);
+    workload = gm_malloc(sizeof(char*)*wsize+1);
     strncpy(workload, (const char*)gearman_job_workload(job), wsize);
     workload[wsize] = '\x0';
     gm_log( GM_LOG_TRACE, "got result %s\n", gearman_job_handle( job ));
     gm_log( GM_LOG_TRACE, "%d +++>\n%s\n<+++\n", strlen(workload), workload );
 
     /* decrypt data */
-    decrypted_data   = malloc(GM_BUFFERSIZE);
+    decrypted_data   = gm_malloc(wsize*2);
     decrypted_data_c = decrypted_data;
 
     if(mod_gm_opt->transportmode == GM_ENCODE_AND_ENCRYPT && mod_gm_opt->accept_clear_results == GM_ENABLED) {
@@ -130,8 +131,9 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
     }
     gm_log( GM_LOG_TRACE, "%d --->\n%s\n<---\n", strlen(decrypted_data), decrypted_data );
 #ifdef GM_DEBUG
-    decrypted_orig   = strdup(decrypted_data);
+    decrypted_orig   = gm_strdup(decrypted_data);
 #endif
+    free(workload);
 
     /*
      * save this result to a file, so when nagios crashes,
@@ -149,15 +151,18 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
 #endif
 
     /* nagios will free it after processing */
-    if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 ) {
+    if ( ( chk_result = ( check_result * )gm_malloc( sizeof *chk_result ) ) == 0 ) {
         *ret_ptr = GEARMAN_WORK_FAIL;
+#ifdef GM_DEBUG
+    free(decrypted_orig);
+#endif
         return NULL;
     }
     init_check_result(chk_result);
     chk_result->scheduled_check     = TRUE;
     chk_result->reschedule_check    = TRUE;
     chk_result->output_file         = 0;
-    chk_result->output_file_fd      = -1;
+    chk_result->output_file_fp      = NULL;
 
     core_start_time.tv_sec          = 0;
     core_start_time.tv_usec         = 0;
@@ -171,10 +176,10 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
 
         if ( !strcmp( key, "output" ) ) {
             if ( value == NULL ) {
-                chk_result->output = strdup("(null)");
+                chk_result->output = gm_strdup("(null)");
             }
             else {
-                chk_result->output = strdup( value );
+                chk_result->output = gm_strdup( value );
             }
         }
 
@@ -182,9 +187,9 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
             break;
 
         if ( !strcmp( key, "host_name" ) ) {
-            chk_result->host_name = strdup( value );
+            chk_result->host_name = gm_strdup( value );
         } else if ( !strcmp( key, "service_description" ) ) {
-            chk_result->service_description = strdup( value );
+            chk_result->service_description = gm_strdup( value );
         } else if ( !strcmp( key, "check_options" ) ) {
             chk_result->check_options = atoi( value );
         } else if ( !strcmp( key, "scheduled_check" ) ) {
@@ -213,6 +218,9 @@ void *get_results( gearman_job_st *job, void *context, size_t *result_size, gear
     if ( chk_result->host_name == NULL || chk_result->output == NULL ) {
         *ret_ptr= GEARMAN_WORK_FAIL;
         gm_log( GM_LOG_ERROR, "discarded invalid job (%s), check your encryption settings\n", gearman_job_handle( job ) );
+#ifdef GM_DEBUG
+    free(decrypted_orig);
+#endif
         return NULL;
     }
 
