@@ -21,11 +21,15 @@ int main (int argc, char **argv, char **env) {
     char *result, *error;
     char cmd[120];
     char hostname[GM_BUFFERSIZE];
+    char cwd[1024];
+    struct stat st;
 
-    plan(60);
+    plan(80);
 
-    /* set hostname */
+    /* set hostname and cwd */
     gethostname(hostname, GM_BUFFERSIZE-1);
+    if(!getcwd(cwd, sizeof(cwd)) != NULL)
+       perror("getcwd() error");
 
     /* create options structure and set debug level */
     mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
@@ -124,11 +128,20 @@ int main (int argc, char **argv, char **env) {
     /*****************************************
      * simple test command 3
      */
-    strcpy(cmd, "/usr/lib/nagios/plugins/check_icmp -H 127.0.0.1");
+    if(stat("/usr/lib/nagios/plugins/check_users", &st) == 0) {
+        strcpy(cmd, "/usr/lib/nagios/plugins/check_users -w 99 -c 99");
+    }
+    else if(stat("/usr/lib64/nagios/plugins/check_users", &st) == 0) {
+        strcpy(cmd, "/usr/lib64/nagios/plugins/check_users -w 99 -c 99");
+    } else {
+        strcpy(cmd, "/bin/false 'no check_users installed...'");
+    }
     rc = run_check(cmd, &result, &error);
     cmp_ok(rc, "==", 0, "pclose for cmd '%s' returned rc %d", cmd, rc);
     rrc = real_exit_code(rc);
     cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "^USERS OK", "returned result string");
+    like(error, "^$", "returned no errors");
     free(result);
     free(error);
 
@@ -365,6 +378,80 @@ int main (int argc, char **argv, char **env) {
     cmp_ok(exec_job->return_code, "==", 0, "cmd '%s' returns rc 0", exec_job->command_line);
     like(exec_job->output, "test plugin OK", "returned result string");
 
+    /*****************************************
+     * restricted paths
+     */
+    char res[150];
+    snprintf(res, 150, "--restrict_path=/tmp/");
+    rc = parse_args_line(mod_gm_opt, res, 0);
+    cmp_ok(rc, "==", GM_OK, "parsed %s option", res);
+    cmp_ok(mod_gm_opt->restrict_path_num, "==", 1, "restricted path is set in opts: %d", mod_gm_opt->restrict_path_num);
+    strcpy(cmd, "./t/ok.pl");
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: restricted paths in affect, but command does not start with an absolute path: ./t/ok.p...", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (2)
+     */
+    strcpy(cmd, "./t/ok.pl; somethingnasty");
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: restricted paths in affect, but command does not start with an absolute path: ./t/ok.p...", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (3)
+     */
+    snprintf(res, 150, "--restrict_path=%s", cwd);
+    rc = parse_args_line(mod_gm_opt, res, 0);
+    cmp_ok(rc, "==", GM_OK, "parsed %s option", res);
+    cmp_ok(mod_gm_opt->restrict_path_num, "==", 2, "restricted path is set in opts: %d", mod_gm_opt->restrict_path_num);
+    snprintf(cmd, sizeof(cmd), "%s/t/ok.pl", cwd);
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "test plugin OK", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (4)
+     */
+    strcpy(cmd, "/forbidden/t/ok.pl");
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: command does not start with any of the restricted paths: /forbidd...", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (5)
+     */
+    snprintf(cmd, sizeof(cmd), "%s/t/ok.pl --test=\"blah\"", cwd);
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 3, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "ERROR: restricted paths in affect, but command contains forbidden character", "returned result string");
+    free(result);
+    free(error);
+
+    /*****************************************
+     * restricted paths (6)
+     */
+    char * restrict_command_characters="$&();<>`|";
+    snprintf(res, 150, "--restrict_command_characters=%s", restrict_command_characters);
+    rc = parse_args_line(mod_gm_opt, res, 0);
+    cmp_ok(rc, "==", GM_OK, "parsed %s option", res);
+    like(mod_gm_opt->restrict_command_characters, restrict_command_characters, "restricted command characters is set in opts: %s", mod_gm_opt->restrict_command_characters);
+    snprintf(cmd, sizeof(cmd), "%s/t/ok.pl --test=\"blah\"", cwd);
+    rrc = real_exit_code(run_check(cmd, &result, &error));
+    cmp_ok(rrc, "==", 0, "cmd '%s' returned rc %d", cmd, rrc);
+    like(result, "test plugin OK", "returned result string");
+    free(result);
+    free(error);
+
     /*****************************************
      * clean up
      */
