@@ -95,14 +95,14 @@ int parse_arguments(int argc, char **argv) {
     int i;
     int verify;
     int errors = 0;
-    mod_gm_opt = malloc(sizeof(mod_gm_opt_t));
+    mod_gm_opt = gm_malloc(sizeof(mod_gm_opt_t));
     set_default_options(mod_gm_opt);
 
     /* special default: encryption disabled */
     mod_gm_opt->encryption = GM_DISABLED;
 
     for(i=1;i<argc;i++) {
-        char * arg   = strdup( argv[i] );
+        char * arg   = gm_strdup( argv[i] );
         char * arg_c = arg;
         if ( !strcmp( arg, "version" ) || !strcmp( arg, "--version" )  || !strcmp( arg, "-V" ) ) {
             print_version();
@@ -209,13 +209,13 @@ int send_result() {
     /* escape newline */
     buf = gm_escape_newlines(mod_gm_opt->message, GM_DISABLED);
     free(mod_gm_opt->message);
-    mod_gm_opt->message = malloc(GM_BUFFERSIZE);
+    mod_gm_opt->message = gm_malloc(GM_BUFFERSIZE);
     snprintf(mod_gm_opt->message, GM_BUFFERSIZE, "%s", buf);
     free(buf);
 
     gm_log( GM_LOG_TRACE, "queue: %s\n", mod_gm_opt->result_queue );
     temp_buffer1[0]='\x0';
-    snprintf( temp_buffer1, sizeof( temp_buffer1 )-1, "type=%s\nhost_name=%s\nstart_time=%i.%i\nfinish_time=%i.%i\nreturn_code=%i\n",
+    snprintf( temp_buffer1, sizeof( temp_buffer1 )-1, "type=%s\nhost_name=%s\nstart_time=%i.%i\nfinish_time=%i.%i\nreturn_code=%i\nsource=send_multi\n",
               mod_gm_opt->active == GM_ENABLED ? "active" : "passive",
               mod_gm_opt->host,
               (int)mod_gm_opt->starttime.tv_sec,
@@ -383,6 +383,7 @@ int read_multi_stream(FILE *stream) {
         }
         gm_log( GM_LOG_TRACE, "\tread %ld bytes, %ld bytes remaining in buffer\n", bytes_read, buflen);
     } while (buflen > 0);
+    buffer[buflen] = '\0';
     return count;
 }
 
@@ -409,7 +410,7 @@ int read_child_check(char *bufstart, char *bufend, struct timeval * end_time) {
     /* service description */
     if ((attribute=read_multi_attribute(bufstart,bufend,"name")) == NULL)
         return 0;
-    mod_gm_opt->service=strdup(attribute);
+    mod_gm_opt->service=gm_strdup(attribute);
     gm_log( GM_LOG_TRACE, "service_description: %s\n", mod_gm_opt->service);
 
     /* return code */
@@ -467,7 +468,7 @@ int read_child_check(char *bufstart, char *bufend, struct timeval * end_time) {
             snprintf( temp_buffer, sizeof( temp_buffer )-1, "%s%s|%s::%s::%s", decode_xml(attribute), decode_xml(error), mod_gm_opt->service, decode_xml(attribute3), decode_xml(attribute2));
         }
     }
-    mod_gm_opt->message=strdup(temp_buffer);
+    mod_gm_opt->message=gm_strdup(temp_buffer);
     gm_log( GM_LOG_TRACE, "mod_gm_opt->message: %s\n", mod_gm_opt->message);
 
     return 1;
